#ifndef ITEM_ID_HPP
#define ITEM_ID_HPP

#include <ItemType.hpp>

template <ItemType item_type>
class ItemIdT
{
 private:
  unsigned int m_id;

 public:
  operator const unsigned int&() const
  {
    return m_id;
  }

  ItemIdT operator++(int)
  {
    ItemIdT item_id(m_id);
    ++m_id;
    return std::move(item_id);
  }

  ItemIdT& operator++()
  {
    ++m_id;
    return *this;
  }

  ItemIdT& operator=(const unsigned int& id)
  {
    m_id = id;
    return *this;
  }

  ItemIdT& operator=(const ItemIdT&) = default;
  ItemIdT& operator=(ItemIdT&&) = default;

  ItemIdT(const unsigned int& id) : m_id{id}{}

  ItemIdT(const ItemIdT&) = default;

  ItemIdT(ItemIdT&&) = default;
  ItemIdT() = default;
  ~ItemIdT() = default;

  // forbidden rules
  template <ItemType other_item_type>
  ItemIdT(const ItemIdT<other_item_type>&)
  {
    static_assert(other_item_type == item_type,
                  "wrong type of item");
  }

  template <ItemType other_item_type>
  ItemIdT& operator=(const ItemIdT<other_item_type>&)
  {
    static_assert(other_item_type == item_type,
                  "wrong type of item");
  }


  template <ItemType other_item_type>
  ItemIdT& operator=(ItemIdT<other_item_type>&&)
  {
    static_assert(other_item_type == item_type,
                  "wrong type of item");
  }

};

using NodeId = ItemIdT<ItemType::node>;
using EdgeId = ItemIdT<ItemType::edge>;
using FaceId = ItemIdT<ItemType::face>;
using CellId = ItemIdT<ItemType::cell>;

#endif // ITEM_ID_HPP
